@@ -387,8 +387,8 @@ class SOCol(object):
                 [self.dbName, self._firebirdType()] + self._extraSQL())
         else:
             return ' '.join(
-                [self.dbName] + [self._firebirdType()[0]] +
-                self._extraSQL() + [self._firebirdType()[1]])
+                [self.dbName] + [self._firebirdType()[0]]
+                + self._extraSQL() + [self._firebirdType()[1]])
 
     def maxdbCreateSQL(self):
         return ' '.join([self.dbName, self._maxdbType()] + self._extraSQL())
@@ -701,7 +701,7 @@ class IntValidator(SOValidator):
             if hasattr(value, attr_name):
                 try:
                     return converter(value)
-                except:
+                except Exception:
                     break
         raise validators.Invalid(
             "expected an int in the IntCol '%s', got %s %r instead" % (
@@ -855,7 +855,7 @@ class FloatValidator(SOValidator):
             if hasattr(value, attr_name):
                 try:
                     return converter(value)
-                except:
+                except Exception:
                     break
         raise validators.Invalid(
             "expected a float in the FloatCol '%s', got %s %r instead" % (
@@ -1213,7 +1213,7 @@ class SetValidator(SOValidator):
             value = (value,)
         try:
             return ",".join(value)
-        except:
+        except Exception:
             raise validators.Invalid(
                 "expected a string or a sequence of strings "
                 "in the SetCol '%s', got %s %r instead" % (
@@ -1284,7 +1284,7 @@ class DateTimeValidator(validators.DateValidator):
                 else:
                     value += '.0'
             return datetime.datetime.strptime(value, self.format)
-        except:
+        except Exception:
             raise validators.Invalid(
                 "expected a date/time string of the '%s' format "
                 "in the DateTimeCol '%s', got %s %r instead" % (
@@ -1345,7 +1345,7 @@ if mxdatetime_available:
                 return DateTime.DateTime(value.year, value.month, value.day,
                                          value.hour, value.minute,
                                          value.second)
-            except:
+            except Exception:
                 raise validators.Invalid(
                     "expected a date/time string of the '%s' format "
                     "in the DateTimeCol '%s', got %s %r instead" % (
@@ -1625,7 +1625,7 @@ class DecimalValidator(SOValidator):
                 value = value.replace(connection.decimalSeparator, ".")
         try:
             return Decimal(value)
-        except:
+        except Exception:
             raise validators.Invalid(
                 "expected a Decimal in the DecimalCol '%s', "
                 "got %s %r instead" % (
@@ -1646,7 +1646,7 @@ class DecimalValidator(SOValidator):
                     value = value.replace(connection.decimalSeparator, ".")
             try:
                 return Decimal(value)
-            except:
+            except Exception:
                 raise validators.Invalid(
                     "can not parse Decimal value '%s' "
                     "in the DecimalCol from '%s'" % (
@@ -1768,12 +1768,12 @@ class BinaryValidator(SOValidator):
             dbName = connection.dbName
             binaryType = connection._binaryType
         if isinstance(value, str):
+            if not PY2 and dbName == "mysql":
+                value = value.encode('ascii', errors='surrogateescape')
             if dbName == "sqlite":
                 if not PY2:
                     value = bytes(value, 'ascii')
                 value = connection.module.decode(value)
-            if dbName == "mysql" and not PY2:
-                value = value.encode('ascii', errors='surrogateescape')
             return value
         if isinstance(value, bytes):
             return value
