@@ -1,54 +1,63 @@
 import atexit
-from cgi import parse_qsl
 import inspect
-import new
-import os
 import sys
+import os
 import threading
 import types
-import urllib
+try:
+    from urlparse import urlparse, parse_qsl
+    from urllib import unquote, quote, urlencode
+except ImportError:
+    from urllib.parse import urlparse, parse_qsl, unquote, quote, urlencode
+
 import warnings
 import weakref
 
-from cache import CacheSet
-import classregistry
-import col
-from converters import sqlrepr
-import main
-import sqlbuilder
-from util.threadinglocal import local as threading_local
+from . import classregistry
+from . import col
+from . import sqlbuilder
+from .cache import CacheSet
+from .compat import PY2, string_type, unicode_type
+from .converters import sqlrepr
+from .events import send, CommitSignal, RollbackSignal
+from .util.threadinglocal import local as threading_local
+
 
 warnings.filterwarnings("ignore", "DB-API extension cursor.lastrowid used")
 
-_connections = {}
 
 def _closeConnection(ref):
     conn = ref()
     if conn is not None:
         conn.close()
 
+
 class ConsoleWriter:
     def __init__(self, connection, loglevel):
         # loglevel: None or empty string for stdout; or 'stderr'
         self.loglevel = loglevel or "stdout"
         self.dbEncoding = getattr(connection, "dbEncoding", None) or "ascii"
+
     def write(self, text):
         logfile = getattr(sys, self.loglevel)
-        if isinstance(text, unicode):
+        if PY2 and isinstance(text, unicode_type):
             try:
                 text = text.encode(self.dbEncoding)
             except UnicodeEncodeError:
-                text = repr(text)[2:-1] # Remove u'...' from the repr
+                text = repr(text)[2:-1]  # Remove u'...' from the repr
         logfile.write(text + '\n')
 
+
 class LogWriter:
     def __init__(self, connection, logger, loglevel):
         self.logger = logger
         self.loglevel = loglevel
         self.logmethod = getattr(logger, loglevel)
+
     def write(self, text):
         self.logmethod(text)
 
+
 def makeDebugWriter(connection, loggerName, loglevel):
     if not loggerName:
         return ConsoleWriter(connection, loglevel)
@@ -56,16 +65,23 @@ def makeDebugWriter(connection, loggerName, loglevel):
     logger = logging.getLogger(loggerName)
     return LogWriter(connection, logger, loglevel)
 
+
 class Boolean(object):
-    """A bool class that also understands some special string keywords (yes/no, true/false, on/off, 1/0)"""
+    """A bool class that also understands some special string keywords
+
+    Understands: yes/no, true/false, on/off, 1/0, case ignored.
+
+    """
     _keywords = {'1': True, 'yes': True, 'true': True, 'on': True,
                  '0': False, 'no': False, 'false': False, 'off': False}
+
     def __new__(cls, value):
         try:
             return Boolean._keywords[value.lower()]
         except (AttributeError, KeyError):
             return bool(value)
 
+
 class DBConnection:
 
     def __init__(self, name=None, debug=False, debugOutput=False,
@@ -111,9 +127,9 @@ class DBConnection:
     def uri(self):
         auth = getattr(self, 'user', '') or ''
         if auth:
-            auth = urllib.quote(auth)
+            auth = quote(auth)
             if self.password:
-                auth = auth + ':' + urllib.quote(self.password)
+                auth = auth + ':' + quote(self.password)
             auth = auth + '@'
         else:
             assert not getattr(self, 'password', None), (
@@ -127,19 +143,22 @@ class DBConnection:
         db = self.db
         if db.startswith('/'):
             db = db[1:]
-        return uri + urllib.quote(db)
+        return uri + quote(db)
 
+    @classmethod
     def connectionFromOldURI(cls, uri):
         return cls._connectionFromParams(*cls._parseOldURI(uri))
-    connectionFromOldURI = classmethod(connectionFromOldURI)
 
+    @classmethod
     def connectionFromURI(cls, uri):
         return cls._connectionFromParams(*cls._parseURI(uri))
-    connectionFromURI = classmethod(connectionFromURI)
 
+    @staticmethod
     def _parseOldURI(uri):
         schema, rest = uri.split(':', 1)
-        assert rest.startswith('/'), "URIs must start with scheme:/ -- you did not include a / (in %r)" % rest
+        assert rest.startswith('/'), \
+            "URIs must start with scheme:/ -- " \
+            "you did not include a / (in %r)" % rest
         if rest.startswith('/') and not rest.startswith('//'):
             host = None
             rest = rest[1:]
@@ -166,9 +185,11 @@ class DBConnection:
             try:
                 port = int(port)
             except ValueError:
-                raise ValueError, "port must be integer, got '%s' instead" % port
+                raise ValueError("port must be integer, "
+                                 "got '%s' instead" % port)
             if not (1 <= port <= 65535):
-                raise ValueError, "port must be integer in the range 1-65535, got '%d' instead" % port
+                raise ValueError("port must be integer in the range 1-65535, "
+                                 "got '%d' instead" % port)
             host = _host
         else:
             port = None
@@ -182,33 +203,23 @@ class DBConnection:
             arglist = arglist.split('&')
             for single in arglist:
                 argname, argvalue = single.split('=', 1)
-                argvalue = urllib.unquote(argvalue)
+                argvalue = unquote(argvalue)
                 args[argname] = argvalue
         return user, password, host, port, path, args
-    _parseOldURI = staticmethod(_parseOldURI)
 
+    @staticmethod
     def _parseURI(uri):
-        protocol, request = urllib.splittype(uri)
+        parsed = urlparse(uri)
+        host, path = parsed.hostname, parsed.path
         user, password, port = None, None, None
-        host, path = urllib.splithost(request)
+        if parsed.username:
+            user = unquote(parsed.username)
+        if parsed.password:
+            password = unquote(parsed.password)
+        if parsed.port:
+            port = int(parsed.port)
 
-        if host:
-            # Python < 2.7 have a problem - splituser() calls unquote() too early
-            #user, host = urllib.splituser(host)
-            if '@' in host:
-                user, host = host.split('@', 1)
-            if user:
-                user, password = [x and urllib.unquote(x) or None for x in urllib.splitpasswd(user)]
-            host, port = urllib.splitport(host)
-            if port: port = int(port)
-        elif host == '':
-            host = None
-
-        # hash-tag is splitted but ignored
-        path, tag = urllib.splittag(path)
-        path, query = urllib.splitquery(path)
-
-        path = urllib.unquote(path)
+        path = unquote(path)
         if (os.name == 'nt') and (len(path) > 2):
             # Preserve backward compatibility with URIs like /C|/path;
             # replace '|' by ':'
@@ -218,13 +229,15 @@ class DBConnection:
             if (path[0] == '/') and (path[2] == ':'):
                 path = path[1:]
 
+        query = parsed.query
+        # hash-tag / fragment is ignored
+
         args = {}
         if query:
             for name, value in parse_qsl(query):
                 args[name] = value
 
         return user, password, host, port, path, args
-    _parseURI = staticmethod(_parseURI)
 
     def soClassAdded(self, soClass):
         """
@@ -248,6 +261,7 @@ class DBConnection:
         for item in cache_set.getAll():
             item.expire()
 
+
 class ConnWrapper(object):
 
     """
@@ -282,11 +296,12 @@ class ConnWrapper(object):
                 "because it takes **kw: %r"
                 % meth)
             takes_conn = 'connection' in args
-            meth.im_func.takes_connection = takes_conn
+            meth.__func__.takes_connection = takes_conn
         if not takes_conn:
             return meth
         return ConnMethodWrapper(meth, self._connection)
 
+
 class ConnMethodWrapper(object):
 
     def __init__(self, method, connection):
@@ -304,6 +319,7 @@ class ConnMethodWrapper(object):
         return '<Wrapped %r with connection %r>' % (
             self._method, self._connection)
 
+
 class DBAPI(DBConnection):
 
     """
@@ -312,14 +328,13 @@ class DBAPI(DBConnection):
 
     ``queryInsertID`` must also be defined.
     """
-
     dbName = None
 
     def __init__(self, **kw):
         self._pool = []
         self._poolLock = threading.Lock()
         DBConnection.__init__(self, **kw)
-        self._binaryType = type(self.module.Binary(''))
+        self._binaryType = type(self.module.Binary(b''))
 
     def _runWithConnection(self, meth, *args):
         conn = self.getConnection()
@@ -341,7 +356,9 @@ class DBAPI(DBConnection):
             if self.debug:
                 s = 'ACQUIRE'
                 if self._pool is not None:
-                    s += ' pool=[%s]' % ', '.join([str(self._connectionNumbers[id(v)]) for v in self._pool])
+                    s += ' pool=[%s]' % ', '.join(
+                        [str(self._connectionNumbers[id(v)])
+                            for v in self._pool])
                 self.printDebug(conn, s, 'Pool')
             return conn
         finally:
@@ -356,14 +373,16 @@ class DBAPI(DBConnection):
             if self._pool is None:
                 s += ' no pooling'
             else:
-                s += ' pool=[%s]' % ', '.join([str(self._connectionNumbers[id(v)]) for v in self._pool])
+                s += ' pool=[%s]' % ', '.join(
+                    [str(self._connectionNumbers[id(v)]) for v in self._pool])
             self.printDebug(conn, s, 'Pool')
         if self.supportTransactions and not explicit:
             if self.autoCommit == 'exception':
                 if self.debug:
                     self.printDebug(conn, 'auto/exception', 'ROLLBACK')
                 conn.rollback()
-                raise Exception, 'Object used outside of a transaction; implicit COMMIT or ROLLBACK not allowed'
+                raise Exception('Object used outside of a transaction; '
+                                'implicit COMMIT or ROLLBACK not allowed')
             elif self.autoCommit:
                 if self.debug:
                     self.printDebug(conn, 'auto', 'COMMIT')
@@ -391,10 +410,10 @@ class DBAPI(DBConnection):
             sep = '->'
             s = repr(s)
         n = self._connectionNumbers[id(conn)]
-        spaces = ' '*(8-len(name))
+        spaces = ' ' * (8 - len(name))
         if self.debugThreading:
             threadName = threading.currentThread().getName()
-            threadName = (':' + threadName + ' '*(8-len(threadName)))
+            threadName = (':' + threadName + ' ' * (8 - len(threadName)))
         else:
             threadName = ''
         msg = '%(n)2i%(threadName)s/%(name)s%(spaces)s%(sep)s %(s)s' % locals()
@@ -408,7 +427,9 @@ class DBAPI(DBConnection):
     def _query(self, conn, s):
         if self.debug:
             self.printDebug(conn, s, 'Query')
-        self._executeRetry(conn, conn.cursor(), s)
+        c = conn.cursor()
+        self._executeRetry(conn, c, s)
+        c.close()
 
     def query(self, s):
         return self._runWithConnection(self._query, s)
@@ -419,6 +440,7 @@ class DBAPI(DBConnection):
         c = conn.cursor()
         self._executeRetry(conn, c, s)
         value = c.fetchall()
+        c.close()
         if self.debugOutput:
             self.printDebug(conn, value, 'QueryAll', 'result')
         return value
@@ -436,6 +458,7 @@ class DBAPI(DBConnection):
         c = conn.cursor()
         self._executeRetry(conn, c, s)
         value = c.fetchall()
+        c.close()
         if self.debugOutput:
             self.printDebug(conn, value, 'QueryAll', 'result')
         return c.description, value
@@ -449,6 +472,7 @@ class DBAPI(DBConnection):
         c = conn.cursor()
         self._executeRetry(conn, c, s)
         value = c.fetchone()
+        c.close()
         if self.debugOutput:
             self.printDebug(conn, value, 'QueryOne', 'result')
         return value
@@ -465,17 +489,19 @@ class DBAPI(DBConnection):
         return Transaction(self)
 
     def queryInsertID(self, soInstance, id, names, values):
-        return self._runWithConnection(self._queryInsertID, soInstance, id, names, values)
+        return self._runWithConnection(self._queryInsertID, soInstance, id,
+                                       names, values)
 
     def iterSelect(self, select):
         return select.IterationClass(self, self.getConnection(),
-                         select, keepConnection=False)
+                                     select, keepConnection=False)
 
     def accumulateSelect(self, select, *expressions):
         """ Apply an accumulate function(s) (SUM, COUNT, MIN, AVG, MAX, etc...)
             to the select object.
         """
-        q = select.queryForSelect().newItems(expressions).unlimited().orderBy(None)
+        q = select.queryForSelect().newItems(expressions).\
+            unlimited().orderBy(None)
         q = self.sqlrepr(q)
         val = self.queryOne(q)
         if len(expressions) == 1:
@@ -512,27 +538,27 @@ class DBAPI(DBConnection):
         return constraints
 
     def createReferenceConstraints(self, soClass):
-        refConstraints = [self.createReferenceConstraint(soClass, column) \
-                          for column in soClass.sqlmeta.columnList \
+        refConstraints = [self.createReferenceConstraint(soClass, column)
+                          for column in soClass.sqlmeta.columnList
                           if isinstance(column, col.SOForeignKey)]
-        refConstraintDefs = [constraint \
-                             for constraint in refConstraints \
+        refConstraintDefs = [constraint for constraint in refConstraints
                              if constraint]
         return refConstraintDefs
 
     def createSQL(self, soClass):
         tableCreateSQLs = getattr(soClass.sqlmeta, 'createSQL', None)
         if tableCreateSQLs:
-            assert isinstance(tableCreateSQLs,(str,list,dict,tuple)), (
+            assert isinstance(tableCreateSQLs, (str, list, dict, tuple)), (
                 '%s.sqlmeta.createSQL must be a str, list, dict or tuple.' %
                 (soClass.__name__))
             if isinstance(tableCreateSQLs, dict):
-                tableCreateSQLs = tableCreateSQLs.get(soClass._connection.dbName, [])
+                tableCreateSQLs = tableCreateSQLs.get(
+                    soClass._connection.dbName, [])
             if isinstance(tableCreateSQLs, str):
                 tableCreateSQLs = [tableCreateSQLs]
             if isinstance(tableCreateSQLs, tuple):
                 tableCreateSQLs = list(tableCreateSQLs)
-            assert isinstance(tableCreateSQLs,list), (
+            assert isinstance(tableCreateSQLs, list), (
                 'Unable to create a list from %s.sqlmeta.createSQL' %
                 (soClass.__name__))
         return tableCreateSQLs or []
@@ -541,13 +567,13 @@ class DBAPI(DBConnection):
         constraints = self.createReferenceConstraints(soClass)
         extraSQL = self.createSQL(soClass)
         createSql = ('CREATE TABLE %s (\n%s\n)' %
-                (soClass.sqlmeta.table, self.createColumns(soClass)))
+                     (soClass.sqlmeta.table, self.createColumns(soClass)))
         return createSql, constraints + extraSQL
 
     def createColumns(self, soClass):
         columnDefs = [self.createIDColumn(soClass)] \
-                     + [self.createColumn(soClass, col)
-                        for col in soClass.sqlmeta.columnList]
+            + [self.createColumn(soClass, col)
+               for col in soClass.sqlmeta.columnList]
         return ",\n".join(["    %s" % c for c in columnDefs])
 
     def createReferenceConstraint(self, soClass, col):
@@ -588,17 +614,17 @@ class DBAPI(DBConnection):
                     self.sqlrepr(so.id)))
 
     def _SO_selectOne(self, so, columnNames):
-        return self._SO_selectOneAlt(so, columnNames, so.q.id==so.id)
-
+        return self._SO_selectOneAlt(so, columnNames, so.q.id == so.id)
 
     def _SO_selectOneAlt(self, so, columnNames, condition):
         if columnNames:
-            columns = [isinstance(x, basestring) and sqlbuilder.SQLConstant(x) or x for x in columnNames]
+            columns = [isinstance(x, string_type)
+                       and sqlbuilder.SQLConstant(x)
+                       or x for x in columnNames]
         else:
             columns = None
-        return self.queryOne(self.sqlrepr(sqlbuilder.Select(columns,
-                                                            staticTables=[so.sqlmeta.table],
-                                                            clause=condition)))
+        return self.queryOne(self.sqlrepr(sqlbuilder.Select(
+            columns, staticTables=[so.sqlmeta.table], clause=condition)))
 
     def _SO_delete(self, so):
         self.query("DELETE FROM %s WHERE %s = (%s)" %
@@ -639,33 +665,37 @@ class DBAPI(DBConnection):
                     self.sqlrepr(secondValue)))
 
     def _SO_columnClause(self, soClass, kw):
-        ops = {None: "IS"}
-        data = {}
+        from . import main
+        data = []
         if 'id' in kw:
-            data[soClass.sqlmeta.idName] = kw.pop('id')
-        for key, col in soClass.sqlmeta.columns.items():
+            data.append((soClass.sqlmeta.idName, kw.pop('id')))
+        for soColumn in soClass.sqlmeta.columnList:
+            key = soColumn.name
             if key in kw:
-                value = kw.pop(key)
-                if col.from_python:
-                    value = col.from_python(value, sqlbuilder.SQLObjectState(soClass, connection=self))
-                data[col.dbName] = value
-            elif col.foreignName in kw:
-                obj = kw.pop(col.foreignName)
+                val = kw.pop(key)
+                if soColumn.from_python:
+                    val = soColumn.from_python(
+                        val,
+                        sqlbuilder.SQLObjectState(soClass, connection=self))
+                data.append((soColumn.dbName, val))
+            elif soColumn.foreignName in kw:
+                obj = kw.pop(soColumn.foreignName)
                 if isinstance(obj, main.SQLObject):
-                    data[col.dbName] = obj.id
+                    data.append((soColumn.dbName, obj.id))
                 else:
-                    data[col.dbName] = obj
+                    data.append((soColumn.dbName, obj))
         if kw:
             # pick the first key from kw to use to raise the error,
-            raise TypeError, "got an unexpected keyword argument(s): %r" % kw.keys()
+            raise TypeError("got an unexpected keyword argument(s): "
+                            "%r" % kw.keys())
 
         if not data:
             return None
         return ' AND '.join(
             ['%s %s %s' %
-             (dbName, ops.get(value, "="), self.sqlrepr(value))
+             (dbName, "IS" if value is None else "=", self.sqlrepr(value))
              for dbName, value
-             in data.items()])
+             in data])
 
     def sqlrepr(self, v):
         return sqlrepr(v, self.dbName)
@@ -682,6 +712,8 @@ class DBAPI(DBConnection):
             return
         self._poolLock.acquire()
         try:
+            if not self._pool:  # _pool could be filled in a different thread
+                return
             conns = self._pool[:]
             self._pool[:] = []
             for conn in conns:
@@ -700,6 +732,16 @@ class DBAPI(DBConnection):
         """
         raise NotImplementedError
 
+    def make_odbc_conn_str(self, odb_source, db, host=None, port=None,
+                           user=None, password=None):
+        odbc_conn_parts = ['Driver={%s}' % odb_source]
+        for odbc_keyword, value in \
+                zip(self.odbc_keywords, (host, port, user, password, db)):
+            if value is not None:
+                odbc_conn_parts.append('%s=%s' % (odbc_keyword, value))
+        self.odbc_conn_str = ';'.join(odbc_conn_parts)
+
+
 class Iteration(object):
 
     def __init__(self, dbconn, rawconn, select, keepConnection=False):
@@ -716,6 +758,9 @@ class Iteration(object):
     def __iter__(self):
         return self
 
+    def __next__(self):
+        return self.next()
+
     def next(self):
         result = self.cursor.fetchone()
         if result is None:
@@ -724,24 +769,29 @@ class Iteration(object):
         if result[0] is None:
             return None
         if self.select.ops.get('lazyColumns', 0):
-            obj = self.select.sourceClass.get(result[0], connection=self.dbconn)
+            obj = self.select.sourceClass.get(result[0],
+                                              connection=self.dbconn)
             return obj
         else:
-            obj = self.select.sourceClass.get(result[0], selectResults=result[1:], connection=self.dbconn)
+            obj = self.select.sourceClass.get(result[0],
+                                              selectResults=result[1:],
+                                              connection=self.dbconn)
             return obj
 
     def _cleanup(self):
         if getattr(self, 'query', None) is None:
             # already cleaned up
             return
-        self.query = None
+        self.cursor.close()
         if not self.keepConnection:
             self.dbconn.releaseConnection(self.rawconn)
-        self.dbconn = self.rawconn = self.select = self.cursor = None
+        self.query = self.dbconn = self.rawconn = \
+            self.select = self.cursor = None
 
     def __del__(self):
         self._cleanup()
 
+
 class Transaction(object):
 
     def __init__(self, dbConnection):
@@ -755,7 +805,9 @@ class Transaction(object):
         self._obsolete = False
 
     def assertActive(self):
-        assert not self._obsolete, "This transaction has already gone through ROLLBACK; begin another transaction"
+        assert not self._obsolete, \
+            "This transaction has already gone through ROLLBACK; " \
+            "begin another transaction"
 
     def query(self, s):
         self.assertActive()
@@ -782,14 +834,19 @@ class Transaction(object):
         # @@: But would it be okay for psycopg, with threadsafety
         # level 2?
         return iter(list(select.IterationClass(self, self._connection,
-                                   select, keepConnection=True)))
+                                               select, keepConnection=True)))
 
     def _SO_delete(self, inst):
         cls = inst.__class__.__name__
-        if not self._deletedCache.has_key(cls):
+        if cls not in self._deletedCache:
             self._deletedCache[cls] = []
         self._deletedCache[cls].append(inst.id)
-        meth = new.instancemethod(self._dbConnection._SO_delete.im_func, self, self.__class__)
+        if PY2:
+            meth = types.MethodType(self._dbConnection._SO_delete.__func__,
+                                    self, self.__class__)
+        else:
+            meth = types.MethodType(self._dbConnection._SO_delete.__func__,
+                                    self)
         return meth(inst)
 
     def commit(self, close=False):
@@ -798,11 +855,13 @@ class Transaction(object):
             return
         if self._dbConnection.debug:
             self._dbConnection.printDebug(self._connection, '', 'COMMIT')
+        self._send_event(CommitSignal)
         self._connection.commit()
-        subCaches = [(sub[0], sub[1].allIDs()) for sub in self.cache.allSubCachesByClassNames().items()]
+        subCaches = [(sub[0], sub[1].allIDs())
+                     for sub in self.cache.allSubCachesByClassNames().items()]
         subCaches.extend([(x[0], x[1]) for x in self._deletedCache.items()])
         for cls, ids in subCaches:
-            for id in ids:
+            for id in list(ids):
                 inst = self._dbConnection.cache.tryGetByName(id, cls)
                 if inst is not None:
                     inst.expire()
@@ -816,15 +875,30 @@ class Transaction(object):
         if self._dbConnection.debug:
             self._dbConnection.printDebug(self._connection, '', 'ROLLBACK')
         subCaches = [(sub, sub.allIDs()) for sub in self.cache.allSubCaches()]
+        self._send_event(RollbackSignal)
         self._connection.rollback()
 
         for subCache, ids in subCaches:
-            for id in ids:
+            for id in list(ids):
                 inst = subCache.tryGet(id)
                 if inst is not None:
                     inst.expire()
         self._makeObsolete()
 
+    def _send_event(self, signal):
+        """
+        Pushes a list of class_names and related ids in cache.
+        :param signal: Type of event signal to use
+        """
+        cached_classes_and_ids = [
+            (class_name, cache.allIDs()) for class_name, cache in
+            self.cache.allSubCachesByClassNames().items()
+        ]
+
+        if cached_classes_and_ids:
+            from .main import sqlmeta  # Import here to avoid circular import
+            send(signal, sqlmeta, cached_classes_and_ids)
+
     def __getattr__(self, attr):
         """
         If nothing else works, let the parent connection handle it.
@@ -834,14 +908,17 @@ class Transaction(object):
         self.assertActive()
         attr = getattr(self._dbConnection, attr)
         try:
-            func = attr.im_func
+            func = attr.__func__
         except AttributeError:
             if isinstance(attr, ConnWrapper):
                 return ConnWrapper(attr._soClass, self)
             else:
                 return attr
         else:
-            meth = new.instancemethod(func, self, self.__class__)
+            if PY2:
+                meth = types.MethodType(func, self, self.__class__)
+            else:
+                meth = types.MethodType(func, self)
             return meth
 
     def _makeObsolete(self):
@@ -856,7 +933,9 @@ class Transaction(object):
     def begin(self):
         # @@: Should we do this, or should begin() be a no-op when we're
         # not already obsolete?
-        assert self._obsolete, "You cannot begin a new transaction session without rolling back this one"
+        assert self._obsolete, \
+            "You cannot begin a new transaction session " \
+            "without rolling back this one"
         self._obsolete = False
         self._connection = self._dbConnection.getConnection()
         self._dbConnection._setAutoCommit(self._connection, 0)
@@ -867,7 +946,11 @@ class Transaction(object):
         self.rollback()
 
     def close(self):
-        raise TypeError('You cannot just close transaction - you should either call rollback(), commit() or commit(close=True) to close the underlying connection.')
+        raise TypeError('You cannot just close transaction - '
+                        'you should either call rollback(), commit() '
+                        'or commit(close=True) '
+                        'to close the underlying connection.')
+
 
 class ConnectionHub(object):
 
@@ -897,7 +980,7 @@ class ConnectionHub(object):
         # I'm a little surprised we have to do this, but apparently
         # the object's private dictionary of attributes doesn't
         # override this descriptor.
-        if (obj is not None) and obj.__dict__.has_key('_connection'):
+        if (obj is not None) and '_connection' in obj.__dict__:
             return obj.__dict__['_connection']
         return self.getConnection()
 
@@ -944,7 +1027,7 @@ class ConnectionHub(object):
         try:
             try:
                 value = func(*args, **kw)
-            except:
+            except Exception:
                 conn.rollback()
                 raise
             else:
@@ -969,6 +1052,7 @@ class ConnectionHub(object):
                                 _set_threadConnection,
                                 _del_threadConnection)
 
+
 class ConnectionURIOpener(object):
 
     def __init__(self):
@@ -978,26 +1062,30 @@ class ConnectionURIOpener(object):
 
     def registerConnection(self, schemes, builder):
         for uriScheme in schemes:
-            assert not self.schemeBuilders.has_key(uriScheme) \
-                   or self.schemeBuilders[uriScheme] is builder, \
-                   "A driver has already been registered for the URI scheme %s" % uriScheme
+            assert uriScheme not in self.schemeBuilders \
+                or self.schemeBuilders[uriScheme] is builder, \
+                "A driver has already been registered " \
+                "for the URI scheme %s" % uriScheme
             self.schemeBuilders[uriScheme] = builder
 
     def registerConnectionInstance(self, inst):
         if inst.name:
-            assert not self.instanceNames.has_key(inst.name) \
-                   or self.instanceNames[inst.name] is cls, \
-                   "A instance has already been registered with the name %s" % inst.name
-            assert inst.name.find(':') == -1, "You cannot include ':' in your class names (%r)" % cls.name
+            assert (inst.name not in self.instanceNames
+                or self.instanceNames[inst.name] is cls  # noqa
+                ), ("A instance has already been registered "
+                "with the name %s" % inst.name)
+            assert inst.name.find(':') == -1, \
+                "You cannot include ':' " \
+                "in your class names (%r)" % cls.name  # noqa
             self.instanceNames[inst.name] = inst
 
     def connectionForURI(self, uri, oldUri=False, **args):
         if args:
             if '?' not in uri:
-                uri += '?' + urllib.urlencode(args)
+                uri += '?' + urlencode(args)
             else:
-                uri += '&' + urllib.urlencode(args)
-        if self.cachedURIs.has_key(uri):
+                uri += '&' + urlencode(args)
+        if uri in self.cachedURIs:
             return self.cachedURIs[uri]
         if uri.find(':') != -1:
             scheme, rest = uri.split(':', 1)
@@ -1008,17 +1096,18 @@ class ConnectionURIOpener(object):
                 conn = connCls.connectionFromURI(uri)
         else:
             # We just have a name, not a URI
-            assert self.instanceNames.has_key(uri), \
-                   "No SQLObject driver exists under the name %s" % uri
+            assert uri in self.instanceNames, \
+                "No SQLObject driver exists under the name %s" % uri
             conn = self.instanceNames[uri]
         # @@: Do we care if we clobber another connection?
         self.cachedURIs[uri] = conn
         return conn
 
     def dbConnectionForScheme(self, scheme):
-        assert self.schemeBuilders.has_key(scheme), (
-               "No SQLObject driver exists for %s (only %s)"
-               % (scheme, ', '.join(self.schemeBuilders.keys())))
+        assert scheme in self.schemeBuilders, (
+            "No SQLObject driver exists for %s (only %s)" % (
+                scheme,
+                ', '.join(self.schemeBuilders.keys())))
         return self.schemeBuilders[scheme]()
 
 TheURIOpener = ConnectionURIOpener()
@@ -1028,12 +1117,13 @@ registerConnectionInstance = TheURIOpener.registerConnectionInstance
 connectionForURI = TheURIOpener.connectionForURI
 dbConnectionForScheme = TheURIOpener.dbConnectionForScheme
 
-# Register DB URI schemas
-import firebird
-import maxdb
-import mssql
-import mysql
-import postgres
-import rdbhost
-import sqlite
-import sybase
+# Register DB URI schemas -- do import for side effects
+# noqa is a directive for flake8 to ignore seemingly unused imports
+from . import firebird  # noqa
+from . import maxdb  # noqa
+from . import mssql  # noqa
+from . import mysql  # noqa
+from . import postgres  # noqa
+from . import rdbhost  # noqa
+from . import sqlite  # noqa
+from . import sybase  # noqa
