@@ -1,8 +1,10 @@
 from itertools import count
-import classregistry
-import events
-import styles
-import sqlbuilder
+from . import boundattributes
+from . import classregistry
+from . import events
+from . import styles
+from . import sqlbuilder
+from .styles import capword
 
 __all__ = ['MultipleJoin', 'SQLMultipleJoin', 'RelatedJoin', 'SQLRelatedJoin',
            'SingleJoin', 'ManyToMany', 'OneToMany']
@@ -10,22 +12,26 @@ __all__ = ['MultipleJoin', 'SQLMultipleJoin', 'RelatedJoin', 'SQLRelatedJoin',
 creationOrder = count()
 NoDefault = sqlbuilder.NoDefault
 
+
 def getID(obj):
     try:
         return obj.id
     except AttributeError:
         return int(obj)
 
+
 class Join(object):
 
     def __init__(self, otherClass=None, **kw):
         kw['otherClass'] = otherClass
         self.kw = kw
         self._joinMethodName = self.kw.pop('joinMethodName', None)
-        self.creationOrder = creationOrder.next()
+        self.creationOrder = next(creationOrder)
 
     def _set_joinMethodName(self, value):
-        assert self._joinMethodName == value or self._joinMethodName is None, "You have already given an explicit joinMethodName (%s), and you are now setting it to %s" % (self._joinMethodName, value)
+        assert self._joinMethodName == value or self._joinMethodName is None, \
+            "You have already given an explicit joinMethodName (%s), " \
+            "and you are now setting it to %s" % (self._joinMethodName, value)
         self._joinMethodName = value
 
     def _get_joinMethodName(self):
@@ -44,9 +50,12 @@ class Join(object):
                               joinMethodName=self._joinMethodName,
                               **self.kw)
 
+
 # A join is separate from a foreign key, i.e., it is
 # many-to-many, or one-to-many where the *other* class
 # has the foreign key.
+
+
 class SOJoin(object):
 
     def __init__(self,
@@ -87,17 +96,47 @@ class SOJoin(object):
 
     def _applyOrderBy(self, results, defaultSortClass):
         if self.orderBy is not None:
-            results.sort(sorter(self.orderBy))
+            doSort(results, self.orderBy)
         return results
 
-def sorter(orderBy):
+
+class MinType(object):
+    """Sort less than everything, for handling None's in the results"""
+    # functools.total_ordering would simplify this
+
+    def __lt__(self, other):
+        if self is other:
+            return False
+        return True
+
+    def __eq__(self, other):
+        return self is other
+
+    def __gt__(self, other):
+        return False
+
+    def __le__(self, other):
+        return True
+
+    def __ge__(self, other):
+        if self is other:
+            return True
+        return False
+
+
+Min = MinType()
+
+
+def doSort(results, orderBy):
     if isinstance(orderBy, (tuple, list)):
         if len(orderBy) == 1:
             orderBy = orderBy[0]
         else:
-            fhead = sorter(orderBy[0])
-            frest = sorter(orderBy[1:])
-            return lambda a, b, fhead=fhead, frest=frest: fhead(a, b) or frest(a, b)
+            # Rely on stable sort results, since this is simpler
+            # than trying to munge everything into a single sort key
+            doSort(results, orderBy[0])
+            doSort(results, orderBy[1:])
+            return
     if isinstance(orderBy, sqlbuilder.DESC) \
        and isinstance(orderBy.expr, sqlbuilder.SQLObjectField):
         orderBy = '-' + orderBy.expr.original
@@ -110,21 +149,17 @@ def sorter(orderBy):
     else:
         reverse = False
 
-    def cmper(a, b, attr=orderBy, rev=reverse):
-        a = getattr(a, attr)
-        b = getattr(b, attr)
-        if rev:
-            a, b = b, a
+    def sortkey(x, attr=orderBy):
+        a = getattr(x, attr)
         if a is None:
-            if b is None:
-                return 0
-            return -1
-        if b is None:
-            return 1
-        return cmp(a, b)
-    return cmper
+            return Min
+        return a
+    results.sort(key=sortkey, reverse=reverse)
+
 
 # This is a one-to-many
+
+
 class SOMultipleJoin(SOJoin):
 
     def __init__(self, addRemoveName=None, **kw):
@@ -139,10 +174,10 @@ class SOMultipleJoin(SOJoin):
             else:
                 name = name + "s"
             self.joinMethodName = name
-        if not addRemoveName:
-            self.addRemoveName = capitalize(self.otherClassName)
-        else:
+        if addRemoveName:
             self.addRemoveName = addRemoveName
+        else:
+            self.addRemoveName = capword(self.otherClassName)
 
     def performJoin(self, inst):
         ids = inst._connection._SO_selectJoin(
@@ -153,7 +188,9 @@ class SOMultipleJoin(SOJoin):
             conn = inst._connection
         else:
             conn = None
-        return self._applyOrderBy([self.otherClass.get(id, conn) for (id,) in ids if id is not None], self.otherClass)
+        return self._applyOrderBy(
+            [self.otherClass.get(id, conn) for (id,) in ids if id is not None],
+            self.otherClass)
 
     def _dbNameToPythonName(self):
         for column in self.otherClass.sqlmeta.columns.values():
@@ -161,9 +198,11 @@ class SOMultipleJoin(SOJoin):
                 return column.name
         return self.soClass.sqlmeta.style.dbColumnToPythonAttr(self.joinColumn)
 
+
 class MultipleJoin(Join):
     baseClass = SOMultipleJoin
 
+
 class SOSQLMultipleJoin(SOMultipleJoin):
 
     def performJoin(self, inst):
@@ -172,13 +211,19 @@ class SOSQLMultipleJoin(SOMultipleJoin):
         else:
             conn = None
         pythonColumn = self._dbNameToPythonName()
-        results = self.otherClass.select(getattr(self.otherClass.q, pythonColumn) == inst.id, connection=conn)
+        results = self.otherClass.select(
+            getattr(self.otherClass.q, pythonColumn) == inst.id,
+            connection=conn)
         return results.orderBy(self.orderBy)
 
+
 class SQLMultipleJoin(Join):
     baseClass = SOSQLMultipleJoin
 
+
 # This is a many-to-many join, with an intermediary table
+
+
 class SORelatedJoin(SOMultipleJoin):
 
     def __init__(self,
@@ -204,7 +249,6 @@ class SORelatedJoin(SOMultipleJoin):
             self.otherColumn = self.soClass.sqlmeta.style.tableReference(
                 otherClass.sqlmeta.table)
 
-
     def hasIntermediateTable(self):
         return True
 
@@ -218,7 +262,9 @@ class SORelatedJoin(SOMultipleJoin):
             conn = inst._connection
         else:
             conn = None
-        return self._applyOrderBy([self.otherClass.get(id, conn) for (id,) in ids if id is not None], self.otherClass)
+        return self._applyOrderBy(
+            [self.otherClass.get(id, conn) for (id,) in ids if id is not None],
+            self.otherClass)
 
     def remove(self, inst, other):
         inst._connection._SO_intermediateDelete(
@@ -236,10 +282,14 @@ class SORelatedJoin(SOMultipleJoin):
             self.otherColumn,
             getID(other))
 
+
 class RelatedJoin(MultipleJoin):
     baseClass = SORelatedJoin
 
+
 # helper classes to SQLRelatedJoin
+
+
 class OtherTableToJoin(sqlbuilder.SQLExpression):
     def __init__(self, otherTable, otherIdName, interTable, joinColumn):
         self.otherTable = otherTable
@@ -251,7 +301,9 @@ class OtherTableToJoin(sqlbuilder.SQLExpression):
         return [self.otherTable, self.interTable]
 
     def __sqlrepr__(self, db):
-        return '%s.%s = %s.%s' % (self.otherTable, self.otherIdName, self.interTable, self.joinColumn)
+        return '%s.%s = %s.%s' % (self.otherTable, self.otherIdName,
+                                  self.interTable, self.joinColumn)
+
 
 class JoinToTable(sqlbuilder.SQLExpression):
     def __init__(self, table, idName, interTable, joinColumn):
@@ -264,7 +316,9 @@ class JoinToTable(sqlbuilder.SQLExpression):
         return [self.table, self.interTable]
 
     def __sqlrepr__(self, db):
-        return '%s.%s = %s.%s' % (self.interTable, self.joinColumn, self.table, self.idName)
+        return '%s.%s = %s.%s' % (self.interTable, self.joinColumn, self.table,
+                                  self.idName)
+
 
 class TableToId(sqlbuilder.SQLExpression):
     def __init__(self, table, idName, idValue):
@@ -278,6 +332,7 @@ class TableToId(sqlbuilder.SQLExpression):
     def __sqlrepr__(self, db):
         return '%s.%s = %s' % (self.table, self.idName, self.idValue)
 
+
 class SOSQLRelatedJoin(SORelatedJoin):
     def performJoin(self, inst):
         if inst.sqlmeta._perConnection:
@@ -293,16 +348,18 @@ class SOSQLRelatedJoin(SORelatedJoin):
                 self.soClass.sqlmeta.table, self.soClass.sqlmeta.idName,
                 self.intermediateTable, self.joinColumn
             ),
-            TableToId(self.soClass.sqlmeta.table, self.soClass.sqlmeta.idName, inst.id),
-        ), clauseTables=(self.soClass.sqlmeta.table, self.otherClass.sqlmeta.table, self.intermediateTable),
-        connection=conn)
+            TableToId(self.soClass.sqlmeta.table, self.soClass.sqlmeta.idName,
+                      inst.id),
+        ), clauseTables=(self.soClass.sqlmeta.table,
+                         self.otherClass.sqlmeta.table,
+                         self.intermediateTable),
+            connection=conn)
         return results.orderBy(self.orderBy)
 
+
 class SQLRelatedJoin(RelatedJoin):
     baseClass = SOSQLRelatedJoin
 
-def capitalize(name):
-    return name[0].capitalize() + name[1:]
 
 class SOSingleJoin(SOMultipleJoin):
 
@@ -324,18 +381,18 @@ class SOSingleJoin(SOMultipleJoin):
             if not self.makeDefault:
                 return None
             else:
-                kw = {self.soClass.sqlmeta.style.instanceIDAttrToAttr(pythonColumn): inst}
-                return self.otherClass(**kw) # instanciating the otherClass with all
+                kw = {self.soClass.sqlmeta.style.
+                      instanceIDAttrToAttr(pythonColumn): inst}
+                # instanciating the otherClass with all
+                return self.otherClass(**kw)
         else:
             return results[0]
 
+
 class SingleJoin(Join):
     baseClass = SOSingleJoin
 
 
-
-import boundattributes
-
 class SOManyToMany(object):
 
     def __init__(self, soClass, name, join,
@@ -383,19 +440,19 @@ class SOManyToMany(object):
         events.listen(self.event_CreateTableSignal,
                       self.otherClass, events.CreateTableSignal)
         self.clause = (
-            (self.otherClass.q.id ==
-             sqlbuilder.Field(self.intermediateTable, self.otherColumn))
+            (self.otherClass.q.id
+                == sqlbuilder.Field(self.intermediateTable, self.otherColumn))
             & (sqlbuilder.Field(self.intermediateTable, self.joinColumn)
-               == self.soClass.q.id))
+                == self.soClass.q.id))
 
     def __get__(self, obj, type):
         if obj is None:
             return self
         query = (
-            (self.otherClass.q.id ==
-             sqlbuilder.Field(self.intermediateTable, self.otherColumn))
+            (self.otherClass.q.id
+                == sqlbuilder.Field(self.intermediateTable, self.otherColumn))
             & (sqlbuilder.Field(self.intermediateTable, self.joinColumn)
-               == obj.id))
+                == obj.id))
         select = self.otherClass.select(query)
         return _ManyToManySelectWrapper(obj, self, select)
 
@@ -409,6 +466,7 @@ class SOManyToMany(object):
             return
         connection._SO_createJoinTable(self)
 
+
 class ManyToMany(boundattributes.BoundFactory):
     factory_class = SOManyToMany
     __restrict_attributes__ = (
@@ -422,6 +480,7 @@ class ManyToMany(boundattributes.BoundFactory):
     otherColumn = None
     createJoinTable = True
 
+
 class _ManyToManySelectWrapper(object):
 
     def __init__(self, forObject, join, select):
@@ -467,6 +526,7 @@ class _ManyToManySelectWrapper(object):
         self.add(obj)
         return obj
 
+
 class SOOneToMany(object):
 
     def __init__(self, soClass, name, join, joinColumn, **attrs):
@@ -497,6 +557,7 @@ class SOOneToMany(object):
         select = self.otherClass.select(query)
         return _OneToManySelectWrapper(obj, self, select)
 
+
 class OneToMany(boundattributes.BoundFactory):
     factory_class = SOOneToMany
     __restrict_attributes__ = (
@@ -506,6 +567,7 @@ class OneToMany(boundattributes.BoundFactory):
     # Default values:
     joinColumn = None
 
+
 class _OneToManySelectWrapper(object):
 
     def __init__(self, forObject, join, select):
