@@ -1,28 +1,35 @@
-from sqlobject.dbconnection import DBAPI
+from getpass import getuser
 import re
 from sqlobject import col
+from sqlobject import dberrors
 from sqlobject import sqlbuilder
-from sqlobject.converters import registerConverter
-from sqlobject.dberrors import *
+from sqlobject.compat import PY2
+from sqlobject.converters import registerConverter, sqlrepr
+from sqlobject.dbconnection import DBAPI
+
 
 class ErrorMessage(str):
     def __new__(cls, e, append_msg=''):
-        obj = str.__new__(cls, e[0] + append_msg)
+        obj = str.__new__(cls, e.args[0] + append_msg)
         if e.__module__ == 'psycopg2':
             obj.code = getattr(e, 'pgcode', None)
             obj.error = getattr(e, 'pgerror', None)
         else:
-            obj.code = obj.error = None
+            obj.code = getattr(e, 'code', None)
+            obj.error = getattr(e, 'error', None)
         obj.module = e.__module__
         obj.exception = e.__class__.__name__
         return obj
 
+
 class PostgresConnection(DBAPI):
 
     supportTransactions = True
     dbName = 'postgres'
     schemes = [dbName, 'postgresql']
 
+    odbc_keywords = ('Server', 'Port', 'UID', 'Password', 'Database')
+
     def __init__(self, dsn=None, host=None, port=None, db=None,
                  user=None, password=None, **kw):
         drivers = kw.pop('driver', None) or 'psycopg'
@@ -31,89 +38,132 @@ class PostgresConnection(DBAPI):
             if not driver:
                 continue
             try:
-                if driver == 'psycopg2':
+                if driver in ('psycopg', 'psycopg2'):
                     import psycopg2 as psycopg
-                elif driver == 'psycopg1':
-                    import psycopg
-                elif driver == 'psycopg':
-                    try:
-                        import psycopg2 as psycopg
-                    except ImportError:
-                        import psycopg
+                    self.module = psycopg
                 elif driver == 'pygresql':
                     import pgdb
                     self.module = pgdb
+                elif driver in ('py-postgresql', 'pypostgresql'):
+                    from postgresql.driver import dbapi20
+                    self.module = dbapi20
+                elif driver == 'pg8000':
+                    import pg8000
+                    self.module = pg8000
+                elif driver == 'pyodbc':
+                    import pyodbc
+                    self.module = pyodbc
+                elif driver == 'pypyodbc':
+                    import pypyodbc
+                    self.module = pypyodbc
+                elif driver == 'odbc':
+                    try:
+                        import pyodbc
+                    except ImportError:
+                        import pypyodbc as pyodbc
+                    self.module = pyodbc
                 else:
-                    raise ValueError('Unknown PostgreSQL driver "%s", expected psycopg2, psycopg1 or pygresql' % driver)
+                    raise ValueError(
+                        'Unknown PostgreSQL driver "%s", '
+                        'expected psycopg, psycopg2, '
+                        'pygresql, pypostgresql, pg8000, '
+                        'odbc, pyodbc or pypyodbc' % driver)
             except ImportError:
                 pass
             else:
                 break
         else:
-            raise ImportError('Cannot find a PostgreSQL driver, tried %s' % drivers)
-        if driver.startswith('psycopg'):
-            self.module = psycopg
-            # Register a converter for psycopg Binary type.
-            registerConverter(type(psycopg.Binary('')),
-                              PsycoBinaryConverter)
+            raise ImportError(
+                'Cannot find a PostgreSQL driver, tried %s' % drivers)
 
+        if driver.startswith('psycopg'):
+            # Register a converter for psycopg Binary type.
+            registerConverter(type(self.module.Binary('')),
+                              PsycoBinaryConverter)
+        elif driver in ('pygresql', 'py-postgresql', 'pypostgresql', 'pg8000'):
+            registerConverter(type(self.module.Binary(b'')),
+                              PostgresBinaryConverter)
+        elif driver in ('odbc', 'pyodbc', 'pypyodbc'):
+            registerConverter(bytearray, OdbcBinaryConverter)
+
+        self.db = db
         self.user = user
+        self.password = password
         self.host = host
         self.port = port
-        self.db = db
-        self.password = password
-        self.dsn_dict = dsn_dict = {}
-        if host:
-            dsn_dict["host"] = host
-        if port:
-            if driver == 'pygresql':
-                dsn_dict["host"] = "%s:%d" % (host, port)
-            else:
-                if psycopg.__version__.split('.')[0] == '1':
+        if driver in ('odbc', 'pyodbc', 'pypyodbc'):
+            self.make_odbc_conn_str(kw.pop('odbcdrv', 'PostgreSQL ANSI'),
+                                    db, host, port, user, password
+                                    )
+            sslmode = kw.pop("sslmode", None)
+            if sslmode:
+                self.odbc_conn_str += ';sslmode=require'
+        else:
+            self.dsn_dict = dsn_dict = {}
+            if host:
+                dsn_dict["host"] = host
+            if port:
+                if driver == 'pygresql':
+                    dsn_dict["host"] = "%s:%d" % (host, port)
+                elif driver.startswith('psycopg') and \
+                        psycopg.__version__.split('.')[0] == '1':
                     dsn_dict["port"] = str(port)
                 else:
                     dsn_dict["port"] = port
-        if db:
-            dsn_dict["database"] = db
-        if user:
-            dsn_dict["user"] = user
-        if password:
-            dsn_dict["password"] = password
-        sslmode = kw.pop("sslmode", None)
-        if sslmode:
-            dsn_dict["sslmode"] = sslmode
-        self.use_dsn = dsn is not None
-        if dsn is None:
-            if driver == 'pygresql':
-                dsn = ''
-                if host:
-                    dsn += host
-                dsn += ':'
-                if db:
-                    dsn += db
-                dsn += ':'
-                if user:
-                    dsn += user
-                dsn += ':'
-                if password:
-                    dsn += password
-            else:
-                dsn = []
-                if db:
-                    dsn.append('dbname=%s' % db)
-                if user:
-                    dsn.append('user=%s' % user)
-                if password:
-                    dsn.append('password=%s' % password)
-                if host:
-                    dsn.append('host=%s' % host)
-                if port:
-                    dsn.append('port=%d' % port)
-                if sslmode:
-                    dsn.append('sslmode=%s' % sslmode)
-                dsn = ' '.join(dsn)
+            if db:
+                dsn_dict["database"] = db
+            if user:
+                dsn_dict["user"] = user
+            if password:
+                dsn_dict["password"] = password
+            sslmode = kw.pop("sslmode", None)
+            if sslmode:
+                dsn_dict["sslmode"] = sslmode
+            self.use_dsn = dsn is not None
+            if dsn is None:
+                if driver == 'pygresql':
+                    dsn = ''
+                    if host:
+                        dsn += host
+                    dsn += ':'
+                    if db:
+                        dsn += db
+                    dsn += ':'
+                    if user:
+                        dsn += user
+                    dsn += ':'
+                    if password:
+                        dsn += password
+                else:
+                    dsn = []
+                    if db:
+                        dsn.append('dbname=%s' % db)
+                    if user:
+                        dsn.append('user=%s' % user)
+                    if password:
+                        dsn.append('password=%s' % password)
+                    if host:
+                        dsn.append('host=%s' % host)
+                    if port:
+                        dsn.append('port=%d' % port)
+                    if sslmode:
+                        dsn.append('sslmode=%s' % sslmode)
+                    dsn = ' '.join(dsn)
+            if driver in ('py-postgresql', 'pypostgresql'):
+                if host and host.startswith('/'):
+                    dsn_dict["host"] = dsn_dict["port"] = None
+                    dsn_dict["unix"] = host
+                else:
+                    if "unix" in dsn_dict:
+                        del dsn_dict["unix"]
+            if driver == 'pg8000':
+                if host and host.startswith('/'):
+                    dsn_dict["host"] = None
+                    dsn_dict["unix_sock"] = host
+                if user is None:
+                    dsn_dict["user"] = getuser()
+            self.dsn = dsn
         self.driver = driver
-        self.dsn = dsn
         self.unicodeCols = kw.pop('unicodeCols', False)
         self.schema = kw.pop('schema', None)
         self.dbEncoding = kw.pop("charset", None)
@@ -122,11 +172,12 @@ class PostgresConnection(DBAPI):
     @classmethod
     def _connectionFromParams(cls, user, password, host, port, path, args):
         path = path.strip('/')
-        if (host is None) and path.count('/'): # Non-default unix socket
+        if (host is None) and path.count('/'):  # Non-default unix socket
             path_parts = path.split('/')
             host = '/' + '/'.join(path_parts[:-1])
             path = path_parts[-1]
-        return cls(host=host, port=port, db=path, user=user, password=password, **args)
+        return cls(host=host, port=port, db=path,
+                   user=user, password=password, **args)
 
     def _setAutoCommit(self, conn, auto):
         # psycopg2 does not have an autocommit method.
@@ -138,61 +189,98 @@ class PostgresConnection(DBAPI):
 
     def makeConnection(self):
         try:
-            if self.use_dsn:
+            if self.driver in ('odbc', 'pyodbc', 'pypyodbc'):
+                self.debugWriter.write(
+                    "ODBC connect string: " + self.odbc_conn_str)
+                conn = self.module.connect(self.odbc_conn_str)
+            elif self.use_dsn:
                 conn = self.module.connect(self.dsn)
             else:
                 conn = self.module.connect(**self.dsn_dict)
-        except self.module.OperationalError, e:
-            raise OperationalError(ErrorMessage(e, "used connection string %r" % self.dsn))
+        except self.module.OperationalError as e:
+            raise dberrors.OperationalError(
+                ErrorMessage(e, "used connection string %r" % self.dsn))
 
         # For printDebug in _executeRetry
         self._connectionNumbers[id(conn)] = self._connectionCount
 
-        if self.autoCommit: self._setAutoCommit(conn, 1)
+        if self.autoCommit:
+            self._setAutoCommit(conn, 1)
         c = conn.cursor()
         if self.schema:
             self._executeRetry(conn, c, "SET search_path TO " + self.schema)
         dbEncoding = self.dbEncoding
         if dbEncoding:
-            self._executeRetry(conn, c, "SET client_encoding TO '%s'" % dbEncoding)
+            if self.driver in ('odbc', 'pyodbc'):
+                conn.setdecoding(self.module.SQL_CHAR, encoding=dbEncoding)
+                conn.setdecoding(self.module.SQL_WCHAR, encoding=dbEncoding)
+                if PY2:
+                    conn.setencoding(str, encoding=dbEncoding)
+                    conn.setencoding(unicode, encoding=dbEncoding)  # noqa
+                else:
+                    conn.setencoding(encoding=dbEncoding)
+            self._executeRetry(conn, c,
+                               "SET client_encoding TO '%s'" % dbEncoding)
+        c.close()
         return conn
 
     def _executeRetry(self, conn, cursor, query):
         if self.debug:
             self.printDebug(conn, query, 'QueryR')
+        dbEncoding = self.dbEncoding
+        if dbEncoding and isinstance(query, bytes) and (
+                self.driver == 'pg8000'):
+            query = query.decode(dbEncoding)
         try:
             return cursor.execute(query)
-        except self.module.OperationalError, e:
-            raise OperationalError(ErrorMessage(e))
-        except self.module.IntegrityError, e:
+        except self.module.OperationalError as e:
+            raise dberrors.OperationalError(ErrorMessage(e))
+        except self.module.IntegrityError as e:
             msg = ErrorMessage(e)
-            if e.pgcode == '23505':
-                raise DuplicateEntryError(msg)
+            if getattr(e, 'code', -1) == '23505' or \
+                    getattr(e, 'pgcode', -1) == '23505' or \
+                    getattr(e, 'sqlstate', -1) == '23505' or \
+                    e.args[0] == '23505':
+                raise dberrors.DuplicateEntryError(msg)
             else:
-                raise IntegrityError(msg)
-        except self.module.InternalError, e:
-            raise InternalError(ErrorMessage(e))
-        except self.module.ProgrammingError, e:
-            raise ProgrammingError(ErrorMessage(e))
-        except self.module.DataError, e:
-            raise DataError(ErrorMessage(e))
-        except self.module.NotSupportedError, e:
-            raise NotSupportedError(ErrorMessage(e))
-        except self.module.DatabaseError, e:
-            raise DatabaseError(ErrorMessage(e))
-        except self.module.InterfaceError, e:
-            raise InterfaceError(ErrorMessage(e))
-        except self.module.Warning, e:
+                raise dberrors.IntegrityError(msg)
+        except self.module.InternalError as e:
+            raise dberrors.InternalError(ErrorMessage(e))
+        except self.module.ProgrammingError as e:
+            msg = ErrorMessage(e)
+            if (
+                (len(e.args) > 2) and (e.args[1] == 'ERROR')
+                and (e.args[2] == '23505')) \
+                    or ((len(e.args) >= 2) and (e.args[1] == '23505')):
+                raise dberrors.DuplicateEntryError(msg)
+            else:
+                raise dberrors.ProgrammingError(msg)
+        except self.module.DataError as e:
+            raise dberrors.DataError(ErrorMessage(e))
+        except self.module.NotSupportedError as e:
+            raise dberrors.NotSupportedError(ErrorMessage(e))
+        except self.module.DatabaseError as e:
+            msg = ErrorMessage(e)
+            if 'duplicate key value violates unique constraint' in msg:
+                raise dberrors.DuplicateEntryError(msg)
+            else:
+                raise dberrors.DatabaseError(msg)
+        except self.module.InterfaceError as e:
+            raise dberrors.InterfaceError(ErrorMessage(e))
+        except self.module.Warning as e:
             raise Warning(ErrorMessage(e))
-        except self.module.Error, e:
-            raise Error(ErrorMessage(e))
+        except self.module.Error as e:
+            raise dberrors.Error(ErrorMessage(e))
 
     def _queryInsertID(self, conn, soInstance, id, names, values):
         table = soInstance.sqlmeta.table
         idName = soInstance.sqlmeta.idName
-        sequenceName = soInstance.sqlmeta.idSequence or \
-                               '%s_%s_seq' % (table, idName)
         c = conn.cursor()
+        if id is None and self.driver in ('py-postgresql', 'pypostgresql'):
+            sequenceName = soInstance.sqlmeta.idSequence or \
+                '%s_%s_seq' % (table, idName)
+            self._executeRetry(conn, c, "SELECT NEXTVAL('%s')" % sequenceName)
+            id = c.fetchone()[0]
         if id is not None:
             names = [idName] + names
             values = [id] + values
@@ -207,6 +295,7 @@ class PostgresConnection(DBAPI):
         self._executeRetry(conn, c, q)
         if id is None:
             id = c.fetchone()[0]
+        c.close()
         if self.debugOutput:
             self.printDebug(conn, id, 'QueryIns', 'result')
         return id
@@ -217,7 +306,7 @@ class PostgresConnection(DBAPI):
             return "%s LIMIT %i" % (query, end)
         if not end:
             return "%s OFFSET %i" % (query, start)
-        return "%s LIMIT %i OFFSET %i" % (query, end-start, start)
+        return "%s LIMIT %i OFFSET %i" % (query, end - start, start)
 
     def createColumn(self, soClass, col):
         return col.postgresCreateSQL()
@@ -240,8 +329,9 @@ class PostgresConnection(DBAPI):
         return 'INT NOT NULL'
 
     def tableExists(self, tableName):
-        result = self.queryOne("SELECT COUNT(relname) FROM pg_class WHERE relname = %s"
-                               % self.sqlrepr(tableName))
+        result = self.queryOne(
+            "SELECT COUNT(relname) FROM pg_class WHERE relname = %s" %
+            self.sqlrepr(tableName))
         return result[0]
 
     def addColumn(self, tableName, column):
@@ -250,7 +340,8 @@ class PostgresConnection(DBAPI):
                     column.postgresCreateSQL()))
 
     def delColumn(self, sqlmeta, column):
-        self.query('ALTER TABLE %s DROP COLUMN %s' % (sqlmeta.table, column.dbName))
+        self.query('ALTER TABLE %s DROP COLUMN %s' % (sqlmeta.table,
+                                                      column.dbName))
 
     def columnsFromSchema(self, tableName, soClass):
 
@@ -307,7 +398,9 @@ class PostgresConnection(DBAPI):
         for isPrimary, indexDef in primaryData:
             match = primaryRE.search(indexDef)
             assert match, "Unparseable contraint definition: %r" % indexDef
-            assert primaryKey is None, "Already found primary key (%r), then found: %r" % (primaryKey, indexDef)
+            assert primaryKey is None, \
+                "Already found primary key (%r), " \
+                "then found: %r" % (primaryKey, indexDef)
             primaryKey = match.group(1)
         if primaryKey is None:
             # VIEWs don't have PRIMARY KEYs - accept help from user
@@ -338,7 +431,8 @@ class PostgresConnection(DBAPI):
                 continue
             if field in keymap:
                 colClass = col.ForeignKey
-                kw = {'foreignKey': soClass.sqlmeta.style.dbTableToPythonClass(keymap[field])}
+                kw = {'foreignKey': soClass.sqlmeta.style.
+                      dbTableToPythonClass(keymap[field])}
                 name = soClass.sqlmeta.style.dbColumnToPythonAttr(field)
                 if name.endswith('ID'):
                     name = name[:-2]
@@ -361,17 +455,17 @@ class PostgresConnection(DBAPI):
         return results
 
     def guessClass(self, t):
-        if t.count('point'): # poINT before INT
+        if t.count('point'):  # poINT before INT
             return col.StringCol, {}
         elif t.count('int'):
             return col.IntCol, {}
         elif t.count('varying') or t.count('varchar'):
             if '(' in t:
-                return col.StringCol, {'length': int(t[t.index('(')+1:-1])}
-            else: # varchar without length in Postgres means any length
+                return col.StringCol, {'length': int(t[t.index('(') + 1:-1])}
+            else:  # varchar without length in Postgres means any length
                 return col.StringCol, {}
         elif t.startswith('character('):
-            return col.StringCol, {'length': int(t[t.index('(')+1:-1]),
+            return col.StringCol, {'length': int(t[t.index('(') + 1:-1]),
                                    'varchar': False}
         elif t.count('float') or t.count('real') or t.count('double'):
             return col.FloatCol, {}
@@ -421,10 +515,12 @@ class PostgresConnection(DBAPI):
         cur = conn.cursor()
         # We must close the transaction with a commit so that
         # the CREATE DATABASE can work (which can't be in a transaction):
-        self._executeRetry(conn, cur, 'COMMIT')
-        self._executeRetry(conn, cur, '%s DATABASE %s' % (op, self.db))
-        cur.close()
-        conn.close()
+        try:
+            self._executeRetry(conn, cur, 'COMMIT')
+            self._executeRetry(conn, cur, '%s DATABASE %s' % (op, self.db))
+        finally:
+            cur.close()
+            conn.close()
 
     def listTables(self):
         return [v[0] for v in self.queryAll(
@@ -444,7 +540,34 @@ class PostgresConnection(DBAPI):
         self._createOrDropDatabase(op="DROP")
 
 
-# Converter for psycopg Binary type.
+# Converter for Binary types
 def PsycoBinaryConverter(value, db):
     assert db == 'postgres'
     return str(value)
+
+
+if PY2:
+    def escape_bytea(value):
+        return ''.join(
+            ['\\' + (x[1:].rjust(3, '0'))
+                for x in (oct(ord(c)) for c in value)]
+        )
+else:
+    def escape_bytea(value):
+        return ''.join(
+            ['\\' + (x[2:].rjust(3, '0'))
+                for x in (oct(ord(c)) for c in value.decode('latin1'))]
+        )
+
+
+def PostgresBinaryConverter(value, db):
+    assert db == 'postgres'
+    return sqlrepr(escape_bytea(value), db)
+
+
+def OdbcBinaryConverter(value, db):
+    assert db == 'postgres'
+    value = bytes(value)
+    if not PY2:
+        value = value.decode('latin1')
+    return value
