@@ -1,14 +1,19 @@
 from datetime import datetime
+from pytest import raises
+
+from sqlobject import KeyCol, MultipleJoin, SQLObject, StringCol, \
+    classregistry, sqlmeta
+from sqlobject.col import use_microseconds
+from sqlobject.tests.dbtest import getConnection, setupClass
+
+
+########################################
+# Dynamic column tests
+########################################
+
+
 now = datetime.now
 
-from sqlobject import *
-from sqlobject.tests.dbtest import *
-from sqlobject import classregistry
-from py.test import raises
-
-########################################
-## Dynamic column tests
-########################################
 
 class Person(SQLObject):
 
@@ -16,12 +21,14 @@ class Person(SQLObject):
         defaultOrder = 'name'
     name = StringCol(length=100, dbName='name_col')
 
+
 class Phone(SQLObject):
 
     class sqlmeta:
         defaultOrder = 'phone'
     phone = StringCol(length=12)
 
+
 class TestPeople:
 
     def setup_method(self, meth):
@@ -34,13 +41,13 @@ class TestPeople:
             Phone(phone=p)
 
     def test_defaultOrder(self):
-        assert (list(Person.select('all')) ==
-                list(Person.select('all', orderBy=Person.sqlmeta.defaultOrder)))
+        assert list(Person.select('all')) == list(
+            Person.select('all', orderBy=Person.sqlmeta.defaultOrder))
 
     def test_dynamicColumn(self):
         nickname = StringCol('nickname', length=10)
         Person.sqlmeta.addColumn(nickname, changeSchema=True)
-        n = Person(name='robert', nickname='bob')
+        Person(name='robert', nickname='bob')
         assert ([p.name for p in Person.select('all')]
                 == ['bob', 'jake', 'jane', 'robert', 'tim'])
         Person.sqlmeta.delColumn(nickname, changeSchema=True)
@@ -55,9 +62,9 @@ class TestPeople:
                 phone.person = Person.selectBy(name='tim')[0]
             else:
                 phone.person = Person.selectBy(name='bob')[0]
-        l = [p.phone for p in Person.selectBy(name='tim')[0].phones]
-        l.sort()
-        assert l == ['555-394-2930', '555-555-5555']
+        _l = [p.phone for p in Person.selectBy(name='tim')[0].phones]
+        _l.sort()
+        assert _l == ['555-394-2930', '555-555-5555']
         Phone.sqlmeta.delColumn(col, changeSchema=True)
         Person.sqlmeta.delJoin(join)
 
@@ -66,16 +73,22 @@ class TestPeople:
             expire = StringCol()
 
     def test_collidingName(self):
-        raises(AssertionError, Person.sqlmeta.addColumn, StringCol(name="name"))
-        raises(AssertionError, Person.sqlmeta.addColumn, StringCol(name="_init"))
-        raises(AssertionError, Person.sqlmeta.addColumn, StringCol(name="expire"))
-        raises(AssertionError, Person.sqlmeta.addColumn, StringCol(name="set"))
+        raises(AssertionError, Person.sqlmeta.addColumn,
+               StringCol(name="name"))
+        raises(AssertionError, Person.sqlmeta.addColumn,
+               StringCol(name="_init"))
+        raises(AssertionError, Person.sqlmeta.addColumn,
+               StringCol(name="expire"))
+        raises(AssertionError, Person.sqlmeta.addColumn,
+               StringCol(name="set"))
         raises(AssertionError, self._test_collidingName)
 
+
 ########################################
-## Auto class generation
+# Auto class generation
 ########################################
 
+
 class TestAuto:
 
     mysqlCreate = """
@@ -92,6 +105,19 @@ class TestAuto:
     """
 
     postgresCreate = """
+    CREATE TABLE auto_test (
+      auto_id SERIAL PRIMARY KEY,
+      first_name VARCHAR(100),
+      last_name VARCHAR(200) NOT NULL,
+      age INT DEFAULT 0,
+      created TIMESTAMP NOT NULL,
+      happy char(1) DEFAULT 'Y' NOT NULL,
+      long_field TEXT,
+      wannahavefun BOOL DEFAULT FALSE NOT NULL
+    )
+    """
+
+    rdbhostCreate = """
     CREATE TABLE auto_test (
       auto_id SERIAL PRIMARY KEY,
       first_name VARCHAR(100),
@@ -123,7 +149,7 @@ class TestAuto:
       first_name VARCHAR(100),
       last_name VARCHAR(200) NOT NULL,
       age INT DEFAULT 0,
-      created VARCHAR(40) NOT NULL,
+      created DATETIME NOT NULL,
       happy char(1) DEFAULT 'Y' NOT NULL,
       long_field TEXT,
       wannahavefun BIT default(0) NOT NULL
@@ -136,7 +162,7 @@ class TestAuto:
       first_name VARCHAR(100),
       last_name VARCHAR(200) NOT NULL,
       age INT DEFAULT 0,
-      created VARCHAR(40) NOT NULL,
+      created DATETIME NOT NULL,
       happy char(1) DEFAULT 'Y' NOT NULL,
       long_field TEXT,
       wannahavefun BIT default(0) NOT NULL
@@ -151,7 +177,7 @@ class TestAuto:
     DROP TABLE auto_test
     """
 
-    sqliteDrop = sybaseDrop = mssqlDrop = postgresDrop
+    sqliteDrop = sybaseDrop = mssqlDrop = rdbhostDrop = postgresDrop
 
     def setup_method(self, meth):
         conn = getConnection()
@@ -165,33 +191,41 @@ class TestAuto:
         dbName = conn.dbName
         dropper = getattr(self, dbName + 'Drop', None)
         if dropper:
-            conn.query(dropper)
+            try:
+                conn.query(dropper)
+            except Exception:  # Perhaps we don't have DROP permission
+                pass
 
     def test_classCreate(self):
         class AutoTest(SQLObject):
             _connection = getConnection()
+
             class sqlmeta(sqlmeta):
                 idName = 'auto_id'
                 fromDatabase = True
+        if AutoTest._connection.dbName == 'mssql':
+            use_microseconds(False)
         john = AutoTest(firstName='john',
                         lastName='doe',
                         age=10,
                         created=now(),
                         wannahavefun=False,
-                        longField='x'*1000)
+                        longField='x' * 1000)
         jane = AutoTest(firstName='jane',
                         lastName='doe',
                         happy='N',
                         created=now(),
                         wannahavefun=True,
-                        longField='x'*1000)
+                        longField='x' * 1000)
         assert not john.wannahavefun
         assert jane.wannahavefun
-        assert john.longField == 'x'*1000
-        assert jane.longField == 'x'*1000
+        assert john.longField == 'x' * 1000
+        assert jane.longField == 'x' * 1000
         del classregistry.registry(
             AutoTest.sqlmeta.registry).classes['AutoTest']
 
         columns = AutoTest.sqlmeta.columns
         assert columns["lastName"].dbName == "last_name"
         assert columns["wannahavefun"].dbName == "wannahavefun"
+        if AutoTest._connection.dbName == 'mssql':
+            use_microseconds(True)
